// Dr Benoit Chaperot 2025, inspired by AI Junkie, Mat Buckland 2002

using System;
using System.Collections.Generic;
using System.Linq;
using UnityEditor;
using UnityEngine;

namespace OkayAI.GA
{
	public partial class GenAlg
	{
		[Serializable]
		public struct Key
		{
			public float time;
			public float value;

			public Key(float inTime, float inValue)
			{
				time = inTime;
				value = inValue;
			}

			public static implicit operator Keyframe(Key key) => new Keyframe(key.time, key.value);
			public static implicit operator Key(Keyframe kf) => new Key() {time = kf.time, value = kf.value};
		}

		static List<Key> CreateFromAnimationCurve(AnimationCurve animationCurve)
		{
			return animationCurve.keys.Select(keyframe => (Key) keyframe).ToList();
		}

		static void CreateFromKeyList(AnimationCurve animationCurve, List<Key> list)
		{
			foreach (Key key in list)
			{
				AddKeyframe(animationCurve, key);
			}
		}

		static void AddKeyframe(AnimationCurve animationCurve, Key key)
		{
			int index = animationCurve.AddKey(key);
			AnimationUtility.SetKeyLeftTangentMode(animationCurve, index, AnimationUtility.TangentMode.Constant);
			AnimationUtility.SetKeyRightTangentMode(animationCurve, index, AnimationUtility.TangentMode.Constant);
		}

		void UpdateCurves()
		{
			AddKeyframe(best, new Key(stats.generation, (float) stats.bestFitness));
			AddKeyframe(average, new Key(stats.generation, (float) stats.averageFitness));
		}
	}
}
