// Dr Benoit Chaperot 2025, inspired by AI Junkie, Mat Buckland 2002

using System;
using System.Collections.Generic;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using Debug = UnityEngine.Debug;

namespace OkayAI.GA
{
	public partial class GenAlg
	{
		[Serializable]
		public struct Save_
		{
			public Parameters parameters;
			public Stats stats;
			public List<Genome> population;
			public List<Key> best;
			public List<Key> average;
		}

		string Filename => parameters.filename;

		public bool Save()
		{
			if (!parameters.saveToFile)
			{
				return false;
			}

			try
			{
				using (FileStream stream = new FileStream(Filename, FileMode.Create, FileAccess.Write))
				{
					Save_ save = new Save_()
					{
						parameters = parameters,
						stats = stats,
						population = CurrentPopulation
					};

					save.best = CreateFromAnimationCurve(best);
					save.average = CreateFromAnimationCurve(average);
					BinaryFormatter formatter = new BinaryFormatter();
					formatter.Serialize(stream, save);
				}
				Debug.Log($"Data written to {Filename} successfully.");
				return true;
			}
			catch (Exception ex)
			{
				Debug.LogError("Error writing to file: " + ex.Message);
				return false;
			}
		}

		public bool Load()
		{
			Debug.Log($"CurrentDirectory: {Directory.GetCurrentDirectory()}");

			if (!parameters.loadFromFile)
			{
				return false;
			}

			if (!File.Exists(Filename))
			{
				Debug.Log($"File {Filename} does not exist.");
				return false;
			}

			try
			{
				using (FileStream stream = File.OpenRead(Filename))
				{
					BinaryFormatter formatter = new BinaryFormatter();
					Save_ save = (Save_) formatter.Deserialize(stream);
					if (save.parameters.populationSize != parameters.populationSize)
					{
						throw new Exception($"Bad population size: was {save.parameters.populationSize} expected {parameters.populationSize}");
					}

					if (save.parameters.chromoLength != parameters.chromoLength)
					{
						throw new Exception($"Bad chromosome length: was {save.parameters.chromoLength} expected {parameters.chromoLength}");
					}

					CurrentPopulation.Clear();
					CurrentPopulation.AddRange(save.population);
					NextPopulation.Clear();
					NextPopulation.AddRange(save.population);

					stats = save.stats;

					CreateFromKeyList(best, save.best);
					CreateFromKeyList(average, save.average);
				}
				Debug.Log($"Data read from {Filename} successfully.");
				return true;
			}
			catch (Exception ex)
			{
				Debug.LogError("Error reading from file: " + ex.Message);
				return false;
			}
		}
	}
}

