// Dr Benoit Chaperot 2025, inspired by AI Junkie, Mat Buckland 2002

using System;
using System.Collections.Generic;
using UnityEngine;

namespace OkayAI.GA
{
	public partial class GenAlg
	{
		public AnimationCurve best = new AnimationCurve();
		public AnimationCurve average = new AnimationCurve();

		[Serializable]
		public class Stats
		{
			public double totalFitness = 0.0;
			public double bestFitness = 0.0;
			public double averageFitness = 0.0;
			public double worstFitness = 0.0;

			// generation counter
			public int generation;

			public void Update(List<Genome> inPopulation)
			{
				totalFitness = 0.0;
				bestFitness = -1e9;
				worstFitness = 1e9;

				for (int i = 0; i < inPopulation.Count; ++i)
				{
					bestFitness = Math.Max(inPopulation[i].fitness, bestFitness);
					worstFitness = Math.Min(inPopulation[i].fitness, worstFitness);
					totalFitness += inPopulation[i].fitness;
				}

				averageFitness = totalFitness / inPopulation.Count;
			}

			public void Tick() => generation++;
		}

		public Stats stats = new Stats();

		readonly List<Genome>[] populations = new[]
		{
			new List<Genome>(), new List<Genome>()
		};

		public int Generation => stats.generation;
		public double BestFitness => stats.bestFitness;
		public double AverageFitness => stats.averageFitness;
		public List<Genome> CurrentPopulation => populations[Generation & 1];
		List<Genome> NextPopulation => populations[(Generation + 1) & 1];
	}
}