// Dr Benoit Chaperot 2025, inspired by AI Junkie, Mat Buckland 2002

using System.Collections.Generic;

namespace OkayAI.NN
{
	public class NeuralNet
	{
		//storage for each layer of neurons including the output layer
		readonly List<NeuronLayer> layers = new List<NeuronLayer>();

		//	this method builds the ANN. The weights are all initially set to
		//	random values -1 < w < 1
		public NeuralNet(int numInputs, int numOutputs, int numHiddenLayers, int neuronsPerHiddenLayer)
		{
			// create the layers of the network
			if (numHiddenLayers > 0)
			{
				// create the first hidden layer
				layers.Add(new NeuronLayer(numInputs, neuronsPerHiddenLayer));

				for (int i = 1; i < numHiddenLayers; ++i)
				{
					layers.Add(new NeuronLayer(neuronsPerHiddenLayer, neuronsPerHiddenLayer));
				}

				// create output layer
				layers.Add(new NeuronLayer(neuronsPerHiddenLayer, numOutputs));
			}

			else
			{
				// create output layer
				layers.Add(new NeuronLayer(numInputs, numOutputs));
			}
		}

		// gets the weights from the NN
		public void GetWeights(List<double> weights)
		{
			weights.Clear();
			weights.Capacity = NumWeights;

			foreach (NeuronLayer layer in layers)
			{
				layer.GetWeights(weights);
			}
		}

		// returns total number of weights in NN
		public int NumWeights
		{
			get
			{
				int numWeights = 0;

				foreach (NeuronLayer layer in layers)
				{
					numWeights += layer.NumWeights;
				}

				return numWeights;
			}
		}

		// puts the weights into the NN
		public void PutWeights(List<double> weights)
		{
			int index = 0;

			foreach (NeuronLayer layer in layers)
			{
				layer.PutWeights(weights, ref index);
			}
		}

		List<double>[] lines = new[]
		{
			new List<double>(), new List<double>()
		};

		long clock = 0;
		void Tick() => clock++;
		public List<double> Inputs => lines[clock & 1];
		List<double> Outputs => lines[(clock + 1) & 1];

		// calculates the outputs from a set of inputs
		public List<double> Update()
		{
			foreach (NeuronLayer layer in layers)
			{
				Outputs.Clear();
				layer.Update(Inputs, Outputs);
				Tick();
			}

			Tick();
			return Outputs;
		}
	};
}