// Dr Benoit Chaperot 2025, inspired by AI Junkie, Mat Buckland 2002

using System.Collections.Generic;

namespace OkayAI.NN
{
	//	class to hold a layer of neurons.
	public class NeuronLayer
	{
		// the layer of neurons
		readonly List<Neuron> neurons = new List<Neuron>();

		public NeuronLayer(int numInputsPerNeuron, int numNeurons)
		{
			for (int i = 0; i < numNeurons; ++i)
			{
				neurons.Add(new Neuron(numInputsPerNeuron));
			}
		}

		public void Update(List<double> inputs, List<double> outputs)
		{
			foreach (Neuron neuron in neurons)
			{
				neuron.Update(inputs, outputs);
			}
		}

		public void GetWeights(List<double> weights)
		{
			foreach (Neuron neuron in neurons)
			{
				weights.AddRange(neuron.GetWeights());
			}
		}

		public int NumWeights
		{
			get
			{
				if (neurons.Count > 0)
				{
					return neurons[0].NumWeights * neurons.Count;
				}

				return 0;
			}
		}

		public void PutWeights(List<double> inWeights, ref int index)
		{
			foreach (Neuron neuron in neurons)
			{
				neuron.PutWeights(inWeights, ref index);
			}
		}
	};
}
