// Dr Benoit Chaperot 2025, inspired by AI Junkie, Mat Buckland 2002

using System.Collections.Generic;
using System;
using Random = UnityEngine.Random;

namespace OkayAI.NN
{
	public class Neuron
	{
		const double Bias = -1.0;
		const double ActivationResponse = 1;
		// returns a random float in the range -1 < n < 1
		static double RandomClamped() { return Random.value - Random.value; }

		// the weights for each input
		readonly List<double> weights = new List<double>();

		// constructor
		public Neuron(int numInputs)
		{
			// we need an additional weight for the bias hence the +1
			for (int i = 0; i < numInputs + 1; i++)
			{
				// set up the weights with an initial random value
				weights.Add(RandomClamped());
			}
		}

		public void Update(List<double> inputs, List<double> outputs)
		{
			double netInput = 0.0;

			int numWeights = NumWeights;

			for (int k = 0; k < numWeights - 1; ++k)
			{
				// sum the weights x inputs
				netInput += weights[k] * inputs[k];
			}

			// add in the bias
			netInput += weights[numWeights - 1] * Bias;

			// we can store the outputs from each layer as we generate them.
			// The combined activation is first filtered through the sigmoid
			// function
			outputs.Add(Sigmoid(netInput, ActivationResponse));
		}

		static double Sigmoid(double netinput, double response)
		{
			return (1.0 / (1.0 + Math.Exp(-netinput / response)));
		}

		public List<double> GetWeights() => weights;
		public int NumWeights => weights.Count;
		public void PutWeights(List<double> inWeights, ref int index)
		{
			int numWeights = NumWeights;
			weights.Clear();
			weights.AddRange(inWeights.GetRange(index, numWeights));
			index += numWeights;
		}
	};
}
