// Dr Benoit Chaperot 2025, inspired by AI Junkie, Mat Buckland 2002

using System;
using OkayAI.GA;
using TMPro;
using UnityEngine;

public partial class Controller : MonoBehaviour
{
	[Serializable]
	public struct Display
	{
		public Camera camera;
		public Transform background;
		public Transform maskL;
		public Transform maskR;

		public TMP_Text generationText;
		public TMP_Text bestFitnessText;
		public TMP_Text averageFitnessText;

		public void OnValidate(float fieldWidth, float fieldHeight)
		{
			camera.transform.position = new Vector3(fieldWidth / 2f, fieldHeight / 2f, -10f);
			camera.orthographicSize = fieldHeight / 2f;
			background.localScale = new Vector3(fieldWidth, fieldHeight, 1f);

			maskL.transform.position = new Vector3(-fieldWidth, 0f, 1f);
			maskL.localScale = new Vector3(fieldWidth, fieldHeight, 1f);

			maskR.transform.position = new Vector3(fieldWidth, 0f, 1f);
			maskR.localScale = new Vector3(fieldWidth, fieldHeight, 1f);
		}

		public void Update(GenAlg ga)
		{
			generationText.text = $"Generation: {ga.Generation}";
			bestFitnessText.text = $"Best Fitness: {(int)ga.BestFitness}";
			averageFitnessText.text = $"Average Fitness: {ga.AverageFitness:f01}";
		}
	}

	public Display display = new Display();

	void UpdateDisplay()
	{
		display.Update(ga);

		if (ga.Generation > 0)
		{
			for (int i = 0; i < gaParameters.eliteNumBest * gaParameters.eliteNumCopies; i++)
			{
				sweepers[i].SetColor(Color.red);
			}
		}
	}
}
