// Dr Benoit Chaperot 2025, inspired by AI Junkie, Mat Buckland 2002

using System.Collections.Generic;
using UnityEngine;
using Random = UnityEngine.Random;

public class BaseSweeper : MonoBehaviour
{
	public Controller controller;
	const float MaxTurnRate = 0.3f;
	public int score;

	protected virtual void MakeDecision(List<Vector2> mines, out float leftTrack, out float rightTrack)
	{
		leftTrack = Random.value;
		rightTrack = Random.value;
	}

	public void UpdateSweeper(List<Vector2> mines)
	{
		MakeDecision(mines, out float leftTrack, out float rightTrack);

		float dRot = Mathf.Clamp(leftTrack - rightTrack, -MaxTurnRate, MaxTurnRate);
		transform.Rotate(Vector3.forward, dRot * Mathf.Rad2Deg);

		float dPos = (leftTrack + rightTrack);

		transform.position += transform.up * dPos;

		WrapAround();
	}

	void OnCollisionEnter2D(Collision2D other)
	{
		Mine mine = other.gameObject.GetComponent<Mine>();
		Debug.Assert(mine);
		mine.ResetPosition();
		score++;
	}

	void WrapAround()
	{
		if (transform.position.x > controller.fieldWidth)
		{
			transform.position -= controller.FieldWidth3;
		}

		if (transform.position.x < 0f)
		{
			transform.position += controller.FieldWidth3;
		}

		if (transform.position.y > controller.fieldHeight)
		{
			transform.position -= controller.FieldHeight3;
		}

		if (transform.position.y < 0f)
		{
			transform.position += controller.FieldHeight3;
		}
	}

	public void SetColor(Color inColor)
	{
		gameObject.GetComponent<SpriteRenderer>().color = inColor;
	}
}
