// Dr Benoit Chaperot 2025, inspired by AI Junkie, Mat Buckland 2002

using System.Collections.Generic;
using OkayAI.NN;
using UnityEngine;

public class SmartSweeper : BaseSweeper
{
	const int NumInputs = 4;
	const int NumOutputs = 2;
	public int numHiddenLayers = 1;
	public int neuronsPerHiddenLayer = 6;

	NeuralNet brain;

	// Start is called before the first frame update
	void Awake()
	{
		brain = new NeuralNet(NumInputs, NumOutputs, numHiddenLayers, neuronsPerHiddenLayer);
	}

	Vector2 GetClosestMine(List<Vector2> mines)
	{
		double closest_so_far = 1e9;
		Vector2 thisPosition = transform.position;
		Vector2 closestMine = Vector2.up;

		//cycle through mines to find closest
		for (int i = 0; i < mines.Count; i++)
		{
			double len_to_object = (mines[i] - thisPosition).SqrMagnitude();

			if (closest_so_far > len_to_object)
			{
				closest_so_far = len_to_object;
				closestMine = mines[i] - thisPosition;
			}
		}

		return closestMine;
	}

	protected override void MakeDecision(List<Vector2> mines, out float leftTrack, out float rightTrack)
	{
		Debug.Assert(brain != null, this);
		List<double> inputs = brain.Inputs;
		Vector2 closestMine = GetClosestMine(mines).normalized;
		Vector2 up = transform.up;
		inputs.Clear();
		inputs.Add(closestMine.x);
		inputs.Add(closestMine.y);
		inputs.Add(up.x);
		inputs.Add(up.y);
		Debug.Assert(inputs.Count == NumInputs);

		List<double> outputs = brain.Update();
		Debug.Assert(outputs.Count == NumOutputs);
		leftTrack = (float) outputs[0];
		rightTrack = (float) outputs[1];
	}

	public int NumWeights => brain.NumWeights;

	public void GetWeights(List<double> weights)
	{
		brain.GetWeights(weights);
	}

	public void PutWeights(List<double> weights)
	{
		brain.PutWeights(weights);
		score = 0;
	}
}
